//
// Created by naushad on 22/11/22.
//

#ifndef SIMBODY_MYROBOT_H
#define SIMBODY_MYROBOT_H
#include "Simbody.h"
#include "URDFReader.h"


class MyRobot : public SimTK::MultibodySystem {

public:
    MyRobot(const std::string &auxDir , const std::string& urdfFileName);

    // Return the parsed-in version of the URDF specification.
    const URDFRobot& getURDFRobot() const {return m_urdfRobot;}
    const SimTK::SimbodyMatterSubsystem& getMatterSubsystem() const {return m_matter;}
    SimTK::SimbodyMatterSubsystem& updMatterSubsystem() {return m_matter;}

    const SimTK::GeneralForceSubsystem& getForceSubsystem() const {return m_forces;}
    SimTK::GeneralForceSubsystem& updForceSubsystem() {return m_forces;}
    /// Realizes the topology and model, and uses the resulting initial state
    /// to perform further internal initialization. Sets up joint limit arrays.
    void initialize(SimTK::State& state);
    SimTK::MobilizedBody& updBody(const std::string& name) {
        return m_urdfRobot.links.updLink(name).masterMobod;
    }
    const SimTK::MobilizedBody& getBody(const std::string& name) const {
        return m_urdfRobot.links.getLink(name).masterMobod;
    }
    const SimTK::MobilizedBody& getEndEffectorBody() const {
        return m_urdfRobot.links.getLink(m_endEffectorLinkName).masterMobod;
    }

    const SimTK::Vec3& getEndEffectorStation() const {
        return m_endEffectorStation;
    }

public:
    std::map<int , SimTK::MobilizedBody> mobodylist;
private:
    SimTK::SimbodyMatterSubsystem               m_matter;
    SimTK::GeneralForceSubsystem                m_forces;
    SimTK::ContactTrackerSubsystem              m_tracker;
    SimTK::CompliantContactSubsystem            m_contact;

    SimTK::Force::Gravity                       m_gravity;
    SimTK::Measure_<SimTK::Vector>::Variable    m_sampledAngles;
    SimTK::Measure_<SimTK::Vector>::Variable    m_sampledRates;
    SimTK::Measure_<SimTK::Transform>::Variable m_sampledPelvisPose;
    SimTK::Measure_<SimTK::Vec3>::Variable      m_sampledEndEffectorPos;
    SimTK::Measure_<SimTK::Real>::Variable      m_qNoise;
    SimTK::Measure_<SimTK::Real>::Variable      m_uNoise;

    SimTK::MultibodyGraphMaker                  m_mbGraph;
    URDFRobot                                   m_urdfRobot;
    std::string                                 m_endEffectorLinkName;
    SimTK::Vec3                                 m_endEffectorStation;

    // Limits on generalized forces and velocities (nu)
    SimTK::Vector                               m_effortLimits;
    SimTK::Vector                               m_speedLimits;

    // Limits on generalized coordinates (nq)
    SimTK::Vector                               m_lowerLimits;
    SimTK::Vector                               m_upperLimits;


};


#endif //SIMBODY_MYROBOT_H
