//
// Created by naushad on 21/11/22.
//

#include "Simbody.h"
#include "shared/TaskSpace.h"
#include "shared/SimbodyExampleHelper.h"
#include "URDFReader.h"
#include <iostream>
#include "MyRobot.h"
using namespace SimTK;
using namespace std;

static const int QNoise=1, UNoise=2; // sliders in the UI



int main(int argc, char **argv) {

    try
    {
        cout << "This is Simbody example '"<< SimbodyExampleHelper::getExampleName() << "'\n";
        cout << "Working dir=" << Pathname::getCurrentWorkingDirectory() << endl;

        const std::string auxDir =
                SimbodyExampleHelper::findAuxiliaryDirectoryContaining
                        ("models/example02.urdf");
        std::cout << "Getting geometry and models from '" << auxDir << "'\n";
        // Set some options.
        const double duration = Infinity; // seconds.


        MyRobot myRobot(auxDir,"example02.urdf");




// Set up visualizer and event handlers.
        Visualizer viz(myRobot);
        viz.setShowFrameRate(true);
        viz.setShowSimTime(true);
        viz.setBackgroundType(SimTK::Visualizer::SolidColor);
        viz.setBackgroundColor(Black);

        MobilizedBody shaft = myRobot.updBody("base_link");



        // Initialize the real robot and other related classes.
        State s;
        myRobot.initialize(s);
        printf("Real robot has %d dofs.\n", s.getNU());



        myRobot.realize(s);
        // Set initial conditions.
        shaft.setQToFitTranslation(s, Vec3(0,0,.5));
        shaft.setUToFitAngularVelocity(s, Pi/180 * Vec3(5, 5, 5)); // 1000 deg/s

//        shaft.setQFromVector(s,Vector(shaft.getNumQ(s), 1.0));

        SemiExplicitEuler2Integrator integ(myRobot);
        integ.setAccuracy(0.001);
        TimeStepper ts(myRobot, integ);
        ts.initialize(s);
        viz.report(ts.getState());
        const double startCPU  = cpuTime(), startTime = realTime();

        // Simulate.
        ts.stepTo(duration);

        std::cout << "CPU time: " << cpuTime() - startCPU << " seconds. "
                  << "Real time: " << realTime() - startTime << " seconds."
                  << std::endl;

        //RungeKuttaMersonIntegrator integ(realRobot);
//        SemiExplicitEuler2Integrator integ(myRobot);
//        integ.setAccuracy(0.001);
//        TimeStepper ts(myRobot, integ);
//        viz.report(s);
//        std::cout<<"This should always appear in your screen"<<'\n';



    }
    catch (const std::exception& e) {
        std::cout << "ERROR: " << e.what() << std::endl;
        return 1;
    }

    return 0;
}